SM3_MAX_MSG_BITLEN: int
SM3_DIGEST_LENGTH: int
SM3_KDF_MAX_LENGTH: int
SM3_MAC_LENGTH: int


class SM3:
    """SM3 算法对象."""

    def __init__(self, data: bytes = b"") -> None:
        """SM3 算法对象.

        Args:
            data: 初始化数据.
        """

    def update(self, data: bytes) -> None:
        """更新内部状态."""

    def digest(self) -> bytes:
        """获取当前状态哈希值."""

    def reset(self) -> None:
        """重置内部状态."""

    def copy(self) -> SM3:
        """拷贝当前对象及其内部状态."""

    def derive_key(self, klen: int) -> bytes:
        """密钥派生函数."""

    def mac(self, key: bytes) -> bytes:
        """消息认证码."""



from typing import Tuple, Optional, Callable

SM2_PARAMS_LENGTH: int
SM2_PARAMS_P: bytes
SM2_PARAMS_A: bytes
SM2_PARAMS_B: bytes
SM2_PARAMS_G: Tuple[bytes, bytes]
SM2_PARAMS_N: bytes

SM2_PCMODE_RAW: int
SM2_PCMODE_COMPRESS: int
SM2_PCMODE_MIX: int

SM2_DEFAULT_UID: bytes
SM2_UID_MAX_LENGTH: int
SM2_ENTITYINFO_LENGTH: int

SM2_SK_LENGTH: int
SM2_PK_HALF_LENGTH: int
SM2_PK_FULL_LENGTH: int

SM2_MSG_MAX_LENGTH: int
SM2_SIGN_R_LENGTH: int
SM2_SIGN_S_LENGTH: int
SM2_SIGNATURE_LENGTH: int

SM2_ENCRYPT_C1_HALF_LENGTH: int
SM2_ENCRYPT_C1_FULL_LENGTH: int
SM2_ENCRYPT_C3_LENGTH: int


class SM2:
    """SM2 算法对象."""

    @staticmethod
    def is_sk_valid(sk: bytes) -> bool:
        """测试私钥是否合法."""

    @staticmethod
    def is_pk_valid(pk: bytes) -> bool:
        """测试公钥是否合法"""

    @staticmethod
    def get_pk(sk: bytes) -> bytes:
        """由私钥得到公钥."""

    def __init__(
        self,
        sk: Optional[bytes] = None,
        pk: Optional[bytes] = None,
        uid: bytes = SM2_DEFAULT_UID,
        *,
        pc_mode: int = SM2_PCMODE_RAW,
        rnd_fn: Optional[Callable[[int], bytes]] = None
    ) -> None:
        """SM2 算法对象.

        Args:
            sk: 私钥.
            pk: 公钥.
            uid: 用户 ID.

        Keyword Args:
            pc_mode: 点压缩模式.
            rnd_fn: 可选的用于产生指定长度随机字节函数.
        """

    def generate_keypair(self) -> Tuple[bytes, bytes]:
        """生成密钥对.

        Returns:
            sk: 私钥.
            pk: 公钥.
        """

    def get_entity_info(self) -> bytes:
        """获取实体信息."""

    def sign_digest(self, digest: bytes) -> bytes:
        """对摘要进行签名."""

    def verify_digest(self, digest: bytes, signature: bytes) -> bool:
        """对摘要及其签名验签."""

    def sign(self, message: bytes) -> bytes:
        """对完整消息进行签名."""

    def verify(self, message: bytes, signature: bytes) -> bool:
        """对完整消息及其签名验签."""



ZUC_KEY_LENGTH: int
ZUC_IV_LENGTH: int
ZUC_WORD_LENGTH: int


class ZUC:
    """ZUC 算法对象."""

    def __init__(self, key: bytes, iv: bytes) -> None:
        """ZUC 算法对象.

        Args:
            key: 密钥.
            iv: 初始化向量.
        """

    def generate(self) -> bytes:
        """生成伪随机字"""

    def copy(self) -> ZUC:
        """拷贝当前对象及其内部状态."""



SM4_KEY_LENGTH: int
SM4_BLOCK_LENGTH: int


class SM4:
    def __init__(self, key: bytes) -> None:
        """SM4 算法对象.

        Args:
            key: 密钥.
        """

    def encrypt(self, block: bytes) -> bytes:
        """加密数据块."""

    def decrypt(self, block: bytes) -> bytes:
        """解密数据块."""



from .core import SM2_PARAMS_LENGTH
from .core import SM2_PARAMS_P
from .core import SM2_PARAMS_A
from .core import SM2_PARAMS_B
from .core import SM2_PARAMS_G
from .core import SM2_PARAMS_N

from .core import SM2_PCMODE_RAW
from .core import SM2_PCMODE_COMPRESS
from .core import SM2_PCMODE_MIX

from .core import SM2_DEFAULT_UID
from .core import SM2_UID_MAX_LENGTH
from .core import SM2_ENTITYINFO_LENGTH

from .core import SM2_SK_LENGTH
from .core import SM2_PK_HALF_LENGTH
from .core import SM2_PK_FULL_LENGTH

from .core import SM2_MSG_MAX_LENGTH
from .core import SM2_SIGN_R_LENGTH
from .core import SM2_SIGN_S_LENGTH
from .core import SM2_SIGNATURE_LENGTH

from .core import SM2_ENCRYPT_C1_HALF_LENGTH
from .core import SM2_ENCRYPT_C1_FULL_LENGTH
from .core import SM2_ENCRYPT_C3_LENGTH

from .core import SM2



from setuptools import Extension, setup

core_module = Extension(
    name="gmalglib.core",
    sources=[
        "src/gmalglib/coremodule.c",
        "src/gmalglib/core/sm3.c",
        "src/gmalglib/core/sm4.c",
        "src/gmalglib/core/zuc.c",
        "src/gmalglib/core/bignum.c",
        "src/gmalglib/core/random.c",
        "src/gmalglib/core/sm2curve.c",
        "src/gmalglib/core/sm2.c",
    ],
    include_dirs=[
        "include",
        "include/python3",
    ],
    depends=[
        "include/gmalglib/sm3.h",
        "include/gmalglib/sm4.h",
        "include/gmalglib/zuc.h",
        "include/gmalglib/bignum.h",
        "include/gmalglib/random.h",
        "include/gmalglib/sm2curve.h",
        "include/gmalglib/sm2.h",
    ]
)

setup(
    ext_modules=[
        core_module
    ]
)
